package de.hdm_stuttgart.mi.sda1.nocast;

import java.io.IOException;
import java.util.Optional;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.Namespace;
import org.jdom2.filter.ElementFilter;
import org.jdom2.input.SAXBuilder;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

/** 
 * How to use XPath expressions to search for &lt;img&gt;
 *  elements in XHTML document instances.
 *  
 */
public class DomXpath {
   private final SAXBuilder builder = new SAXBuilder();

   /** 
    * Read an XHTML document and search for images. The image filenames will be
    * written to standard out.
    * 
    * @param xhtmlFilename The XHTML's filename to be examined.
    * @return A CSV containing all image references separated by ", "
    * @throws JDOMException General parsing fault
    * @throws IOException file access problem
    */
   public Optional<String> process(final String xhtmlFilename) 
		   throws JDOMException, IOException {

      final Document htmlInput = builder.build(xhtmlFilename);

      final XPathExpression<Element> xpath = 
            XPathFactory.instance().compile(
                  "//html:img",    // See last argument for prefix definition "html"
                  new ElementFilter(),
                  null,
                  Namespace.getNamespace("html", "http://www.w3.org/1999/xhtml"));

      return xpath.evaluate(htmlInput).
      stream().
      map(img -> img.getAttributeValue("src")).
      reduce((l,r) -> l.concat(", ").concat(r));
   }
}